/* Подключение датчика DS18B20 по pin0 (RxD)
индикация температуры на 2х разрядном 
7-сегм.индикаторе
программная динамическая развертка по сегментам 3мс на сегмент
8 сегментов - 2 разряда (резисторы 100ом) Общий Анод */

#define DIG1 9
#define DIG2 A0
#define s_A 6
#define s_B 5
#define s_C 13
#define s_D 10
#define s_E 12
#define s_F 7
#define s_G 8
#define s_H 11
#define Takt 3  //мс на каждый сегмент (индикация)

#define T_DISP 1000 //длительность индикации параметров (мс)
#define T_PAUSE 600 //длительность пауз между индикацией параметров (мс)

byte var_i = 5;   //переключатель индикации (сначала номер версии)
unsigned long tLoop_Disp = 0; //время показа параметра температуры (мс)
unsigned long tLoop_Tek = 0; //текущее время (мс)

int dig1 = 0; //левый
int dig2 = 0; //правый
byte pnt = 0; //точка меж разрядами (1 - горит)
byte pntEnd = 0; //точка в конце (1 - горит)
int var_Disp = 4; //индицируемое значение (сначала номер версии)

//посегментная развертка 0 в разряде - погашено!
byte segA = 0; //развертка по сегментная, А
byte segB = 0; //развертка по сегментная, B
byte segC = 0; //развертка по сегментная, C
byte segD = 0; //развертка по сегментная, D
byte segE = 0; //развертка по сегментная, E
byte segF = 0; //развертка по сегментная, F
byte segG = 0; //развертка по сегментная, G
byte segH = 0; //развертка по сегментная, H

#include <OneWire.h>  //для термометра DS18B20

float celsius;   //результат измерения температуры
OneWire  ds(0);  // on pin 0 (a 4.7K resistor is necessary)

void setup() {                
  pinMode(s_A, OUTPUT); pinMode(s_B, OUTPUT);
  pinMode(s_C, OUTPUT); pinMode(s_D, OUTPUT);
  pinMode(s_E, OUTPUT); pinMode(s_F, OUTPUT);
  pinMode(s_G, OUTPUT); pinMode(s_H, OUTPUT);
  pinMode(DIG1, OUTPUT);
  pinMode(DIG2, OUTPUT);
  Clean();
  digitalWrite(DIG1,HIGH);
  digitalWrite(DIG2,HIGH);
}

void loop() {
  tLoop_Tek = millis();

  if (tLoop_Tek > tLoop_Disp) {
  switch(var_i) {
    case 1: {
//       Measure_Temperature_Start();  //запустим измерение температуры
//       delay(1000);
       Measure_Temperature_Read(); //считаем температуру с датчика 
       // * отображение температуры
       var_Disp = int(celsius * 10);
       pntEnd = 1; //точка в конце - признак температуры
       tLoop_Disp = tLoop_Tek + T_DISP; //1 сек на показ температуры
       var_i ++; }
    break;
    case 2: {
       Measure_Temperature_Start();  //запустим измерение температуры
//       var_Disp = 1000;       //гасим индикатор
       tLoop_Disp = tLoop_Tek + T_PAUSE; //пауза
       var_i ++; }
    break;
    case 3: {
       Measure_Temperature_Read(); //считаем температуру с датчика 
       // * отображение температуры
       var_Disp = int(celsius * 10);
       pntEnd = 0; //без точки в конце - признак температуры 
       tLoop_Disp = tLoop_Tek + T_DISP; //1 сек на показ влажности
       var_i ++; }
    break;
    case 4: {
       Measure_Temperature_Start();  //запустим измерение температуры
       tLoop_Disp = tLoop_Tek + T_PAUSE; //пауза
       var_i = 1; }
    break;
    case 5: {
       Measure_Temperature_Start();  //запустим измерение температуры
       //* отображение номера версии программы
       pntEnd = 0; //без точки в конце
       tLoop_Disp = tLoop_Tek + T_DISP; //1 сек на показ 
       var_i = 1; }
    break;
    default: 
     {   //если сбой
       Measure_Temperature_Start();  //запустим измерение температуры
       tLoop_Disp = 0; //выход в режим 
       var_i = 1; }
    break;
  }
  }

  DisplayMath(var_Disp);  //индикация
  DigToSegmRaz();  //раскидать цифры по сегментам развертки
  DisplayShow();
}

void DisplayMath(int data) {
  //data должна быть в 10 раз больше индицируемого, например, для 27 будет «2.7»
//при data 1000 и больше индикатор погашен
//при data -100 и менее на индикаторе «- .»
//при data от 0 до -100 отображается «минус» и разряд десятков, например -27 будет «-2»
  pnt = 0; //изначально точка погашена
  dig1 = dig2 = 0;
  if (data < 100) {
      if (data < 0){
          if (data <= -100) {
             // - гасим правый индикатор, вкл точку и «минус»
             dig1 = 11;  //знак «минус»
             dig2 = 10;
             pnt = 0;
             pntEnd = 1;
             }
          else {
             dig1 = 11; //знак «минус»
             dig2 = int((0-data)/10); //целые цифры
             }
          }
     else{
          while (data >= 10) {
             data -= 10;
             dig1++;
             }
          dig2 = data;
          pnt = 1;  //вкл точку меж разрядами
         }
     }
  else {
    if (data < 1000) {
       data = data / 10;
       while (data >= 10) {
          data -= 10;
          dig1++;
       }
       dig2 = data;
    }
    else {  //при 1000 и выше - гасим индикатор
      dig1 = 10;
      dig2 = 11;
      pnt = 0;
      pntEnd = 0;
    }
  }  
}

void DisplayShow() {
  digitalWrite(s_A,LOW);  //включаем линию сегментов А
  ShowSeg(segA);
  delay(Takt);
  digitalWrite(s_A,HIGH);  //выключаем линию сегментов А
  digitalWrite(s_B,LOW);   //вкл В
  ShowSeg(segB);
  delay(Takt);
  digitalWrite(s_B,HIGH);  //выключаем линию сегментов В
  digitalWrite(s_C,LOW);   //вкл C
  ShowSeg(segC);
  delay(Takt);
  digitalWrite(s_C,HIGH);  //выключаем линию сегментов C
  digitalWrite(s_D,LOW);   //вкл D
  ShowSeg(segD);
  delay(Takt);
  digitalWrite(s_D,HIGH);  //выключаем линию сегментов D
  digitalWrite(s_E,LOW);   //вкл E
  ShowSeg(segE);
  delay(Takt);
  digitalWrite(s_E,HIGH);  //выключаем линию сегментов E
  digitalWrite(s_F,LOW);   //вкл F
  ShowSeg(segF);
  delay(Takt);
  digitalWrite(s_F,HIGH);  //выключаем линию сегментов F
  digitalWrite(s_G,LOW);   //вкл G
  ShowSeg(segG);
  delay(Takt);
  digitalWrite(s_G,HIGH);  //выключаем линию сегментов G
  digitalWrite(s_H,LOW);   //вкл H
  ShowSeg(segH);
  delay(Takt);
  digitalWrite(s_H,HIGH);  //выключаем линию сегментов H
}

void ShowSeg(byte digit)  {
  switch(digit) {
    case 1: {
       digitalWrite(DIG1,LOW);    //правый разряд включить
       digitalWrite(DIG2,HIGH); }
    break;
    case 2: {
       digitalWrite(DIG1,HIGH);    //левый разряд включить
       digitalWrite(DIG2,LOW); }
    break;
    case 3: {
       digitalWrite(DIG1,HIGH);    //оба разряда включить
       digitalWrite(DIG2,HIGH); }
    break;
    default: 
     {   //case 0:
       digitalWrite(DIG1,LOW);    //оба разряда гасим
       digitalWrite(DIG2,LOW); }
    break;
  }
}  

//раскидать цифры по сегментам развертки
void DigToSegmRaz() {
  //для левого разряда
  switch(dig1) {
    case 0: {
      segA = B10; segB = B10; segC = B10; segD = B10;
      segE = B10; segF = B10; segG = B00;    }
    break;
    case 1: {
      segA = B00; segB = B10; segC = B10; segD = B00;
      segE = B00; segF = B00; segG = B00;    }
    break;
    case 2: {
      segA = B10; segB = B10; segC = B00; segD = B10;
      segE = B10; segF = B00; segG = B10;    }
    break;
    case 3: {
      segA = B10; segB = B10; segC = B10; segD = B10;
      segE = B00; segF = B00; segG = B10;    }
    break;
    case 4: {
      segA = B00; segB = B10; segC = B10; segD = B00;
      segE = B00; segF = B10; segG = B10;    }
    break;
    case 5: {
      segA = B10; segB = B00; segC = B10; segD = B10;
      segE = B00; segF = B10; segG = B10;    }
    break;
    case 6: {
      segA = B10; segB = B00; segC = B10; segD = B10;
      segE = B10; segF = B10; segG = B10;    }
    break;
    case 7: {
      segA = B10; segB = B10; segC = B10; segD = B00;
      segE = B00; segF = B00; segG = B00;    }
    break;
    case 8: {
      segA = B10; segB = B10; segC = B10; segD = B10;
      segE = B10; segF = B10; segG = B10;    }
    break;
    case 9: {
      segA = B10; segB = B10; segC = B10; segD = B10;
      segE = B00; segF = B10; segG = B10;    }
    break;
    case 10: { //гашение
      segA = B00; segB = B00; segC = B00; segD = B00;
      segE = B00; segF = B00; segG = B00;    }
    break;
    case 11: { //знак «минус»
      segA = B00; segB = B00; segC = B00; segD = B00;
      segE = B00; segF = B00; segG = B10;    }
    break;
  }
  //для правого разряда
  switch(dig2) {
    case 0: {
      segA = segA | 1; segB = segB | 1; segC = segC | 1; segD = segD | 1;
      segE = segE | 1; segF = segF | 1;   }
    break;
    case 1: {
      segB = segB | 1; segC = segC | 1;   }
    break;
    case 2: {
      segA = segA | 1; segB = segB | 1; segD = segD | 1;
      segE = segE | 1; segG = segG | 1;    }
    break;
    case 3: {
      segA = segA | 1; segB = segB | 1; segC = segC | 1; segD = segD | 1;
      segG = segG | 1;    }
    break;
    case 4: {
      segB = segB | 1; segC = segC | 1;
      segF = segF | 1; segG = segG | 1;    }
    break;
    case 5: {
      segA = segA | 1; segC = segC | 1; segD = segD | 1;
      segF = segF | 1; segG = segG | 1;    }
    break;
    case 6: {
      segA = segA | 1; segC = segC | 1; segD = segD | 1;
      segE = segE | 1; segF = segF | 1; segG = segG | 1;    }
    break;
    case 7: {
      segA = segA | 1; segB = segB | 1; segC = segC | 1;   }
    break;
    case 8: {
      segA = segA | 1; segB = segB | 1; segC = segC | 1; segD = segD | 1;
      segE = segE | 1; segF = segF | 1; segG = segG | 1;    }
    break;
    case 9: {
      segA = segA | 1; segB = segB | 1; segC = segC | 1; segD = segD | 1;
      segF = segF | 1; segG = segG | 1;    }
    break;
    case 10: { //гашение 
    }
    break;
    case 11: { //знак «минус»
      segG = segG | 1;    }
    break;
  }
  if (pnt == 1)  segH = B10;
  else  segH = B00;
  if (pntEnd == 1)  segH = segH | 1;
}

void Clean() {  //все гасим (очистка)
    digitalWrite(s_A,HIGH); digitalWrite(s_B,HIGH);  //катоды +
    digitalWrite(s_C,HIGH); digitalWrite(s_D,HIGH);
    digitalWrite(s_E,HIGH); digitalWrite(s_F,HIGH);
    digitalWrite(s_G,HIGH); digitalWrite(s_H,HIGH);
    digitalWrite(DIG1,HIGH); digitalWrite(DIG2,HIGH);   //аноды + 
}

void Measure_Temperature_Start() {
//for 1-wire  (ds18b20)
  ds.reset();
  ds.write(0xCC);        //skip rom (не читать память с адресом)
  ds.write(0x44, 1);     // start conversion, with parasite power on at the end
//после запуска надо подождать, чтоб замер прошел – примерно секунду 
}


void Measure_Temperature_Read() {
//for 1-wire  (ds18b20)
  byte i;
  byte data[12];
  ds.reset();
  ds.write(0xCC);            //skip rom (не читать память с адресом)
  ds.write(0xBE);         // Read Scratchpad

  for ( i = 0; i < 9; i++) {           // we need 9 bytes
    data[i] = ds.read();
  }

  // Convert the data to actual temperature
  // because the result is a 16 bit signed integer, it should
  // be stored to an "int16_t" type, which is always 16 bits
  // even when compiled on a 32 bit processor.
  int16_t raw = (data[1] << 8) | data[0];
  celsius = (float)raw / 16.0;
//  celsius = raw;   //все 16 бит без коррекции 
}

